-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
M.type = "auxiliary"
M.relevantDevice = nil

local htmlTexture = require("htmlTexture")

local min = math.min
local max = math.max

local gaugesScreenName = nil
local htmlPath = nil

local updateTimer = 0
local updateFPS = 15
local invFPS = 1 / updateFPS

local function getIsEngineRunning()
  return (electrics.values["engineRunning"] or 0) > 0.5
end

local function updateGFX(dt)
  --screen update
  updateTimer = updateTimer + dt
  if updateTimer > invFPS then
    --get gear
    local gear = "" -- none by default
    if getIsEngineRunning() then 
      if electrics.values.gearIndex == 0 then
          gear = "N"
      elseif electrics.values.gearIndex < 0 then
          gear = "R"
       else
          gear = tostring(electrics.values.gearIndex)
       end
    end
    
   --set gear
    htmlTexture.call(gaugesScreenName, "setGearFromLua", {value=gear})
  end
end

local function init(jbeamData)
  gaugesScreenName = jbeamData.materialName
  htmlPath = jbeamData.htmlPath
  local width = jbeamData.textureWidth or 128
  local height = jbeamData.textureHeight or 128

  if not gaugesScreenName then
    log("E", "fs65transhtml", "Got no material name for the texture, can't display anything...")
    M.updateGFX = nop
  else
    if htmlPath then
      htmlTexture.create(gaugesScreenName, htmlPath, width, height, updateFPS, "automatic")
      htmlTexture.call(gaugesScreenName, "setGearFromLua", {value="N"})
    else
      log("E", "fs65transhtml", "Got no html path for the texture, can't display anything...")
      M.updateGFX = nop
    end
  end
end

M.init = init
M.updateGFX = updateGFX

return M
