local M = {}

M.enabled = true
local beams = {}
local poseElapsedTimes = {}
local poseRestLengths = {}

local function resetPoseElapsedTimes()
  local newTable = {}
  for k,_ in pairs(poseElapsedTimes) do newTable[k] = 0 end
  poseElapsedTimes = newTable
end

local function init()
	beams = {}
  for _,beam in pairs(v.data.beams) do
      if beam.poseBeamTime and type(beam.poseBeamTime) == 'number' 
      and beam.poseBeamTarget and type(beam.poseBeamTarget) == 'number' then
          table.insert(beams, beam)
          poseElapsedTimes[beam.cid] = 0
          poseRestLengths[beam.cid] = obj:getBeamRestLength(beam.cid)
      end
  end
end

local function reset()
  resetPoseElapsedTimes()
end

--
local function lerp(a,b,t) return a * (1-t) + b * t end

local function updateGFX(dt)
    if not M.enabled then return end
    for _,beam in pairs(beams) do
            local targetTime = beam.poseBeamTime
            local currentTime = poseElapsedTimes[beam.cid]
            if currentTime <= targetTime then
                poseElapsedTimes[beam.cid] = currentTime + dt
                local lerpTarget = poseElapsedTimes[beam.cid] / targetTime
                local lengthAtRest = poseRestLengths[beam.cid]
                local finalLength = lengthAtRest * beam.poseBeamTarget
                local setLength = lerp(lengthAtRest, finalLength, lerpTarget)
                --print("cid=" .. tostring(beam.cid) .. ", lt=" .. tostring(lerpTarget) .. ", lar=" .. tostring(lengthAtRest) .. ", fl=" .. tostring(finalLength) ..  ", sl=" .. tostring(setLength))
                obj:setBeamLength(beam.cid, setLength)
          else
              obj:setBeamSpringDamp(beam.cid, 0, 0, 0, 0)
          end
    end
end

local function nop()
end

-- controller api --
M.init = init
M.reset = reset
M.updateGFX = updateGFX
return M