local M = {}

local railNodeMap = {}
local railNodeMapByCid = {}
local slidenodeBrokenMap = {}
local slidenodeBrokenMapByCid = {}
local slidenodeNodeIds = {}
local slidenodes = {}

--public api
local function isSlidenodeBroken(cid) 
	local onRail = slidenodeBrokenMapByCid[cid]
	if onRail == nil then return false else return not onRail end 
end

--
local function constructMaps()
	railNodeMap = {}
	slidenodeBrokenMap = {}
	slidenodeNodeIds = {}
	railNodeMapByCid = {}
	slidenodeBrokenMapByCid = {}
	
	--create rail node map
	for key,rail in pairs(v.data.rails) do
		if key ~= "cids" then
			if rail["links:"] == nil then 
				print("NIL links on rail " .. key .. "?")
				dump(rail)
			else
				local map = {}
				for _, nodeId in pairs(rail["links:"]) do
					table.insert(map, v.data.nodes[nodeId])
				end
				railNodeMap[key] = map
				railNodeMapByCid[rail.cid] = map
			end
		end
		
	end
	
	--next, broken map
	for key,sn in pairs(v.data.slidenodes) do
		local node = v.data.nodes[sn.id]
		slidenodeBrokenMap[node.cid] = false
		table.insert(slidenodeNodeIds, node.cid)
		table.insert(slidenodes, sn)
	end
end



local function isNodeOnLine(nodeCid, lineStartNodeCid, lineEndNodeCid, tolerance)
	local lineStart = obj:getNodePosition(lineStartNodeCid)
	local lineEnd = obj:getNodePosition(lineEndNodeCid)
	local nodePos = obj:getNodePosition(nodeCid)

	local line = (lineEnd - lineStart)
	local length = line:length()
	line:normalize()
	
	local v = (nodePos - lineStart)
	local d = v:dot(line)
	
	-- easy route
	if d < 0 or d > length then return false end
	
	-- tolerance route
	local closestPoint = vec3(lineStart.x + (line.x * d), lineStart.y + (line.y * d), lineStart.z + (line.z * d))
	local dist = closestPoint:distance(nodePos)
	return dist < tolerance
end

local function isnan(x) return x ~= x end

local function getNodeFromTable(name)
  for k,v in pairs(v.data.nodes) do
    if v.name == name then return v end
  end
  return nil
end


local function init()
	constructMaps()
end

local function reset()
  init()
end

local function clamp(val, min, max)
   if val < min then return min end
   if val > max then return max end
   return val
end


--updatePhysics()
local function updateGFX(dt)
	for idx,cid in ipairs(slidenodeNodeIds) do
		local slidenode = slidenodes[idx]
		local railName = slidenode.railName
		local railNodes = railNodeMap[railName]
		local nodeId = slidenodeNodeIds[idx]
		
		local onRail = false
		for i=1, #railNodes - 1 do
			local rn1 = railNodes[i]
			local rn2 = railNodes[i+1]
			if isNodeOnLine(nodeId, rn1.cid, rn2.cid, 0.1) then onRail = true end
		end
		
		--if onRail == false then print("ONRAIL FALSE FUR " .. tostring(nodeId)) end
		slidenodeBrokenMap[idx] = onRail
		slidenodeBrokenMapByCid[nodeId] = onRail
	end
end

local function updateGFX(dt)

end

local function nop()
end

-- controller api --
M.init = init
M.reset = reset
M.update = updatePhysics
M.updateGFX = updateGFX

-- public api --
M.isSlidenodeBroken = isSlidenodeBroken

return M