local M = {}

local airbrakeAlarmSound= nil
local emergencyDoorAlarmSound = nil
local airHornSound = nil
local sfxprofilecounter = 0 -- local counter to enumerate the profiles without collisions, do not reset ever

local lastCamPos = nil
local isCameraInside = false

local emergencyDoorHingeCid = -1
local emergencyDoorLatchCid = -1
local airhornBreakCid = -1
local driverNodeCid = -1

local hasAirDoors = false
local doorOpenSound = nil
local doorCloseSound = nil
local timeSinceDoorSoundPlayed = 0
local lastDoorValue = false
local doorSoundPlaying = false
local doorBeamCid = nil

local soundObj = {}
soundObj.__index = soundObj

abs = math.abs
min = math.min
max = math.max

function soundObj:setVolumePitch(vol, pitch, color, texture)
  if vol < 0.01 then
    vol = 0
  end
  if vol == 0 then
    if self.lastVol == 0 then
      return
    end
    color, texture = color or 0, texture or 0
  else
    color, texture = color or 0, texture or 0
    if abs(pitch - self.lastPitch) < 0.001 and max(abs(vol - self.lastVol), abs(color - self.lastColor), abs(texture - self.lastTexture)) < 0.01 then
      return
    end
  end
  self.lastVol = vol
  self.lastPitch = pitch
  self.lastColor = color
  self.lastTexture = texture
  obj:setVolumePitchCT(self.obj, vol, pitch, color, texture)
end

function soundObj:mute()
  self:setVolumePitch(0, self.lastPitch)
end

function soundObj:unmute()
  self:setVolumePitch(1, self.lastPitch)
end

function soundObj:setVolumeOP(vol)
   --setVolume One Pitch
  self:setVolumePitch(vol, 1.0)
end

local function newSoundObj(sndObj)
  if sndObj == nil then
    return nil
  end
  local data = {obj = sndObj, lastVol = 0, lastPitch = 0, lastColor = 0, lastTexture = 0}
  setmetatable(data, soundObj)
  return data
end

local function createSoundObj(filename, description, SFXProfileName, nodeID)
  local snd = obj:createSFXSource(filename, description, SFXProfileName, nodeID)
  if snd == nil then
    log("W", "cb40_custom_sounds.createSoundObj", "failed to create sfx source: " .. SFXProfileName .. " from file " .. filename .. " with description " .. description)
    return nil
  end
  return newSoundObj(snd)
end


local function getNextProfile()
  sfxprofilecounter = sfxprofilecounter + 1
  return "cb40_LuaSoundProfile" .. sfxprofilecounter .. "_" .. os.time()
end

local function findBeamCid(name)
  for _,beam in pairs(v.data.beams) do
      if beam.name == name then return beam.cid end
  end
  return -1
end
local function findNodeCid(name)
  for _,node in pairs(v.data.nodes) do
      if node.name == name then return node.cid end
  end
  return -1
end

local function onInit()
    driverNodeCid = findNodeCid("driver")
    emergencyDoorHingeCid = findBeamCid("emergencyDoorHinge")
    emergencyDoorLatchCid = findBeamCid("emergencyDoorLatch")
    doorBeamCid = findBeamCid("doorHydro")
    
    local dashNodeCid = findNodeCid("dsh1l")
    if dashNodeCid < 0 then dashNodeCid = findNodeCid("bd0cl") end
    
    --
    local airHornNode = findNodeCid("airhn0r")
    airhornBreakCid = -1
    if airHornNode >= 0 then
        airHornSound = airHornSound or createSoundObj("event:>Vehicle>Electrics>Horns>TRUCK", "AudioDefaultLoop3D", getNextProfile(), airHornNode)
        airHornSound:setVolumePitch(0, 1)
        airhornBreakCid = findBeamCid("airhornBreak")
    end
    
    --
    local airbrakeAlarmSoundName = "cb40_alarm"
    local partConfig = partmgmt.state.config.parts
    if partConfig ~= nil then
      if partConfig["cb40_dash"] == "cb40_dash_02" then 
          airbrakeAlarmSoundName = "cb40_buzz"
      end
    end
    
    airbrakeAlarmSound = airbrakeAlarmSound or createSoundObj("vehicles/cb40/custom_sounds/" .. airbrakeAlarmSoundName .. ".wav", "AudioDefaultLoop3D", getNextProfile(), dashNodeCid)
    emergencyDoorAlarmSound = emergencyDoorAlarmSound or createSoundObj("vehicles/cb40/custom_sounds/cb40_emerg_exit_alarm_2.wav", "AudioDefaultLoop3D", getNextProfile(), dashNodeCid)
    airbrakeAlarmSound:setVolumePitch(0, 1)
    emergencyDoorAlarmSound:setVolumePitch(0, 1)
    
    --
    local partConfig = partmgmt.state.config.parts
    if partConfig ~= nil then
      local doorType = partConfig["cb40_sidedoor_type"] or ""
      hasAirDoors = doorType == "cb40_sidedoor_type_air"
      --print("hasAirDoors is " .. tostring(hasAirDoors))
      --print("doorType is " .. doorType)
    end
    
    if hasAirDoors then
      local doorNode = findNodeCid("bd1drr")
      doorOpenSound = obj:createSFXSource( "event:>Vehicle>Pneumatics>Door_Open", "AudioDefaultLoop3D", "airdooropen", doorNode)
      doorCloseSound = obj:createSFXSource( "event:>Vehicle>Pneumatics>Door_Close", "AudioDefaultLoop3D", "airdoorclose", doorNode)
    end
end

local function stopDoorSounds()
    obj:stopSFX(doorCloseSound)
    obj:stopSFX(doorOpenSound)
    doorSoundPlaying = false
end

local function onReset()
  isCameraInside = false
  lastDoorValue = false
  
  if hasAirDoors then
    timeSinceDoorSoundPlayed = 999 --make sound playable again
    stopDoorSounds()
  end
end

local function getSideDoorsOpen()
    return (electrics.values["open_doors"] or 0) > 0.5
end

local function getIsEngineRunning()
  return (electrics.values["engineRunning"] or 0) > 0.5
end

local function shouldPlayEmergencyDoorAlarm()
  return obj:beamIsBroken(emergencyDoorLatchCid) and not obj:beamIsBroken(emergencyDoorHingeCid)
end

local function updateCameraShit()
  local camPos = obj:getCameraPosition()
  local driverPos = obj:getNodePosition(driverNodeCid)
  local myPos = obj:getPosition()
  local combinedPos = myPos + driverPos
  
  local camDistFromDriver = (combinedPos - camPos):length()
  isCameraInside = camDistFromDriver <= 0.6
end

local airHornVolSmooth = 0
local function updateGFX(dt)
  updateCameraShit()

  -- update door stuff  
  if hasAirDoors then
    timeSinceDoorSoundPlayed = timeSinceDoorSoundPlayed + dt
    local doorValue = getSideDoorsOpen()
    if timeSinceDoorSoundPlayed >= 2  and doorSoundPlaying then
        stopDoorSounds()
    end
    
    local doorBrokenOff = false
    if doorBeamCid ~= nil then doorBrokenOff = obj:beamIsBroken(doorBeamCid) end
    
    if doorValue ~= lastDoorValue and timeSinceDoorSoundPlayed > 2 and not doorBrokenOff then
        if doorValue then
            obj:setVolume(doorOpenSound, 1)
            obj:cutSFX(doorOpenSound)
            obj:playSFX(doorOpenSound)
        else
            obj:setVolume(doorCloseSound, 1)
            obj:cutSFX(doorCloseSound)
            obj:playSFX(doorCloseSound)
        end
        doorSoundPlaying = true
        timeSinceDoorSoundPlayed = 0
    end
    lastDoorValue = doorValue
  end
  
  -- exterior sounds
  if airHornSound ~= nil then
    local airHornBroken = obj:beamIsBroken(airhornBreakCid)
    if electrics.values.airhorn and electrics.values.airhorn > 0.5 and airHornSound ~= nil and not airHornBroken then
        airHornVolSmooth = math.min(1.5, airHornVolSmooth + (dt * 20))
        airHornSound:setVolumePitch(airHornVolSmooth, 1)
    elseif airHornSound ~= nil then
        airHornVolSmooth = max(0, airHornVolSmooth - (dt * 30))
        airHornSound:setVolumePitch(airHornVolSmooth, 1)
    end
  end
  
  -- interior sounds
  if not isCameraInside then
    airbrakeAlarmSound:setVolumeOP(0)
    emergencyDoorAlarmSound:setVolumeOP(0)
    return
  end
  
  if shouldPlayEmergencyDoorAlarm() then
    emergencyDoorAlarmSound:setVolumeOP(1)
  else
    emergencyDoorAlarmSound:setVolumeOP(0)
  end
  
  if getIsEngineRunning() then
    local minAirPressure = min(electrics.values["cb40_airp_real"], electrics.values["cb40_airs_real"])
    if minAirPressure >= 75 then
      airbrakeAlarmSound:setVolumeOP(0)
    else
      airbrakeAlarmSound:setVolumeOP(1)
    end
  else 
    airbrakeAlarmSound:setVolumeOP(0)
  end
end

-- public interface
M.onInit      = onInit
M.onReset     = onReset
M.updateGFX = updateGFX

M.cameraPosCallback = cameraPosCallback

return M