local M = {}

-- decorative air gauges
M.forceWatertemp = nil
M.airTestMode = false
local airpRand = 0
local airsRand = 0
local airDecayRate = 1/5333
local airGainRate = 0.1
local airTargetP = 0.7
local airTargetS = 0.72

-- oil gauges
local engine = nil
local oilPsiBase = 53
local oilPsiGainPerJ = 0.0021
local oilPressurePsi = 0
local oilPressurePsiSmoother = nil

local function getIsEngineRunning()
  return (electrics.values["engineRunning"] or 0) > 0.5
end

local function clamp(value, min, max)
    if value < min then return min end
    if value > max then return max end
    return value
end


local function onInit()
  electrics.values["cb40_nansignal"] = 0
  
  electrics.values["cb40_oilpressure"] = 0
  electrics.values["cb40_watertemp"] = 0
  electrics.values["cb40_airp"] = 0
  electrics.values["cb40_airs"] = 0
  
  electrics.values["cb40_airp_real"] = 0
  electrics.values["cb40_airs_real"] = 0
  
  electrics.values["gear_M_mod"] = 0
  
  local r1 = (math.random() - 0.5) * 0.2
  local r2 = (math.random() - 0.5) * 0.2
  airpRand = r1 + 1.0
  airsRand = r2 + 1.0
  
  engine = powertrain.getDevice("mainEngine")
  oilPressurePsiSmoother = newExponentialSmoothing(60)
end

local function onReset()
	--onInit()
  if M.airTestMode then
    electrics.values["cb40_airp"] = 0
    electrics.values["cb40_airs"] = 0
  end
end

local function isNaN( v ) return type( v ) == "number" and v ~= v end 

local function updateGFX(dt)
  if M.forceWatertemp ~= nil and type(M.forceWatertemp) == 'number' then
      electrics.values["cb40_watertemp"] = M.forceWatertemp
  else
      local watertemp = electrics.values["watertemp"] or 0
      if watertemp > 37.7778 then
        electrics.values["cb40_watertemp"] = (watertemp - 37.7778) / (121.111 - 37.7778)
      else
        electrics.values["cb40_watertemp"] = 0
      end
  end
  
  -- update nan signal
  local nansignalValue = 0
  if isNaN(electrics.values.wheelspeed or 0) then nansignalValue = 1 end
  electrics.values["cb40_nansignal"] = nansignalValue
  
  -- update oil pressure
  local oilPressureTarget  = 0
  if getIsEngineRunning() then oilPressureTarget = oilPsiBase end
  if engine ~= nil and engine.thermals ~= nil and  getIsEngineRunning() then
    oilPressureTarget = oilPressureTarget + ((engine.thermals.debugData.engineThermalData.energyToOil / dt) * oilPsiGainPerJ)
  end
  
  if oilPressureTarget > oilPressurePsi then oilPressurePsi = min(oilPressureTarget, oilPressurePsi + (dt * 15)) end
  if oilPressureTarget<  oilPressurePsi then oilPressurePsi = max(oilPressureTarget, oilPressurePsi - (dt * 15)) end
  oilPressurePsiSmoother:set(oilPressurePsi)
  electrics.values["cb40_oilpressure"] = oilPressurePsiSmoother:value() / 100

  -- update shifter
  local gearM = electrics.values["gear_M"]  or 0
  local gearMod = 0
  if gearM ~= 0 then
      --special reverse logic
      if gearM < 0 then 
          gearMod = 0 
      else
          gearMod = gearM % 2
       end
      if gearMod == 0 then gearMod = -1 end
  end
  electrics.values["gear_M_mod"] = gearMod
  
  --update airp and airs
  local airp = electrics.values["cb40_airp"] 
  local airs =  electrics.values["cb40_airs"] 
  local engineOn = getIsEngineRunning()
  if engineOn then
      local gainRateMultiplier =  min(1, (electrics.values.rpm or 0) / 500)
      airp = clamp(airp + (airGainRate * airpRand * dt * gainRateMultiplier), 0, airTargetP)
      airs = clamp(airs + (airGainRate * airsRand * dt * gainRateMultiplier), 0, airTargetS)
  else
      airp = clamp(airp - (airDecayRate * airpRand * dt), 0, airTargetP)
      airs = clamp(airs - (airDecayRate * airsRand * dt), 0, airTargetS)
  end
  electrics.values["cb40_airp"]  = airp
   electrics.values["cb40_airs"]  = airs
   electrics.values["cb40_airp_real"] = airp * 150
   electrics.values["cb40_airs_real"] = airs * 150
end

-- public interface
M.onInit      = onInit
M.onReset     = onReset
M.updateGFX = updateGFX

return M