local M = {}

local enabled = false
local slotID = -1
local editProp = nil

local baseRotation = {x=0, y=0, z=0}

-- command editing mode
M.cycleXSpeed = 0
M.cycleYSpeed = 0
M.cycleZSpeed = 0

-- keybind editing mode
local editAxis = "x"
local editAxisSpeed = 0
local function setEditAxis(axis)
    gui.message("Edit axis now set to " .. axis, 3, "propedit")
    editAxis = axis
end

local function editAxisStart(speed)
  editAxisSpeed = speed
end

local function editAxisEnd()
  editAxisSpeed = 0
end

local function onInit()
  if v.data.props == nil then return end
  for _,prop in pairs(v.data.props) do
    if prop.edit ~= nil and prop.edit then 
          print("Found edit prop")
          enabled = true 
          slotID = prop.slotID
          editProp = prop
          baseRotation = prop.baseRotation
    end
  end
end

local function onReset()
	onInit()
end

local function updateGFX(dt)
    --get prop
    if not enabled or slotID<  0 then return end
    if editProp == nil or editProp.propObj == nil then return end
    local prop = editProp.propObj 
    
    -- keybind
    if editAxisSpeed < 0.01 or editAxisSpeed > 0.01 then
        baseRotation[editAxis] = (baseRotation[editAxis] + (dt * 0.0174533 * editAxisSpeed) ) % 6.28319
    end
    
    -- cycle
    if M.cycleXSpeed ~= 0 then
      baseRotation.x = (baseRotation.x + (dt * 0.0174533 * M.cycleXSpeed) ) % 6.28319
    end
    if M.cycleYSpeed ~= 0 then
      baseRotation.y = (baseRotation.y + (dt * 0.0174533 * M.cycleYSpeed) ) % 6.28319
    end
    if M.cycleZSpeed ~= 0 then
      baseRotation.z = (baseRotation.z + (dt * 0.0174533 * M.cycleZSpeed) ) % 6.28319
    end
    
    -- set rotation
    editProp.baseRotation = tableToFloat3(baseRotation)
    prop:setBaseRotation(float3_2_Vector3(editProp.baseRotation))
end

-- public functions
local function getBaseRotation()
   print("x=" .. tostring(baseRotation.x * 57.2958) .. ", y=" .. tostring(baseRotation.y * 57.2958) .. ",z=" ..tostring(baseRotation.z * 57.2958))
end

local function setX(degrees)
  baseRotation.x = degrees * 0.0174533
end

local function setY(degrees)
  baseRotation.y = degrees * 0.0174533
end

local function setZ(degrees)
  baseRotation.z = degrees * 0.0174533
end

-- public interface
M.onInit      = onInit
M.onReset     = onReset
M.updateGFX = updateGFX

M.getBaseRotation = getBaseRotation
M.setX = setX
M.setY = setY
M.setZ = setZ

M.setEditAxis = setEditAxis
M.editAxisStart = editAxisStart
M.editAxisEnd = editAxisEnd

return M