-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

--LOCAL ENUM
local AXIS_X = 0
local AXIS_Y = 1
local AXIS_Z = 2

local function computeNodesAngle(sn, en, axis)
  local cx = 0
  local cy = 0
  local ex = 0
  local ey = 0
  if axis == 0 then
    --x axis (from front)
    cx = sn.x
    cy = sn.z
    ex = en.x
    ey = en.z
  elseif axis == 1 then
    --y axis (from side)
    cx = sn.y
    cy = sn.z
    ex = en.y
    ey = en.z
  elseif axis == 2 then
    --z axis (from top)
    cx = sn.x
    cy = sn.y
    ex = en.x
    ey = en.y
  end

  local dy = ey - cy;
  local dx = ex - cx
  local theta = math.atan2(dy, dx)
  theta = theta * (180 / math.pi)
  if theta < 0 then theta = 360 + theta end
  return theta - 180;
end


local function getNode(name)
    for k,node in pairs(v.data.nodes) do
        if node.name == name then return node end
    end
end

local function getNodePos(name)
    local node = getNode(name)
    if node == nil then return end
    
    return obj:getNodePosition(node.cid)
end

local function getYRefRotation()
    return computeNodesAngle(getNodePos("fr4bl"), getNodePos("fr5bl"), 2) + 90
end
local function getZRefRotation()
    return computeNodesAngle(getNodePos("fr4bl"), getNodePos("fr4tl"), 0) - 90
end


local function dump()
    local refYAngle = getYRefRotation()
    local refZAngle = getZRefRotation()

    --
    local lfHubOuterNode = getNodePos("knlon")
    local lfHubInnerNode = getNodePos("knlin")
    local lfToeAngle = computeNodesAngle(lfHubOuterNode, lfHubInnerNode, AXIS_Z) - refYAngle
    print("LF TOE ANGLE " .. tostring(lfToeAngle))

    local rfHubOuterNode = getNodePos("knron")
    local rfHubInnerNode = getNodePos("knrin")
    local rfToeAngle = computeNodesAngle(rfHubInnerNode, rfHubOuterNode, AXIS_Z) - refYAngle
    print("RF TOE ANGLE " .. tostring(rfToeAngle))

end

M.dump = dump
return M
